﻿Option Explicit On
Option Infer On
Option Strict On

Imports System.Xml

Public Class Form1
    Private Sub Form1_Load() Handles MyBase.Load
        ' Create data.
        Dim all_customers As New List(Of Customer)
        all_customers.Add(New Customer() With {.FirstName = "Ann", .LastName = "Archer", .Balance = 100D})
        all_customers.Add(New Customer() With {.FirstName = "Ben", .LastName = "Best", .Balance = -24.54D})
        all_customers.Add(New Customer() With {.FirstName = "Carly", .LastName = "Cant", .Balance = 62.4D})
        all_customers.Add(New Customer() With {.FirstName = "Dan", .LastName = "Dump", .Balance = 117.95D})
        all_customers.Add(New Customer() With {.FirstName = "Edna", .LastName = "Ever", .Balance = -192.75D})
        all_customers.Add(New Customer() With {.FirstName = "Frank", .LastName = "Fix", .Balance = -150.9D})

        Dim txt As String = ""

        ' Select all customers.
        'Dim x_all As XElement =
        '    <AllCustomers>
        '        <%= From cust In all_customers
        '            Select New XElement("Customer",
        '            New XAttribute("FirstName", cust.FirstName),
        '            New XAttribute("LastName", cust.LastName),
        '            New XText(cust.Balance.ToString("0.00")))
        '        %>
        '    </AllCustomers>

        ' A different format.
        Dim x_all As XElement =
            <AllCustomers>
                <%= From cust In all_customers
                    Select <Customer
                               FirstName=<%= cust.FirstName %>
                               LastName=<%= cust.LastName %>
                               >
                               <%= cust.Balance.ToString("0.00") %>
                           </Customer>
                %>
            </AllCustomers>
        txt &= "LINQ All Customers:" & vbCrLf & x_all.ToString() & vbCrLf & vbCrLf

        ' Separate customers with positive and negative balances.
        Dim separated As XElement =
            <AllCustomers>
                <PositiveBalances>
                    <%= From cust In x_all.Descendants("Customer")
                        Where CDec(cust.Value) >= 0
                        Order By CDec(cust.Value) Descending
                        Select New XElement("Customer",
                        New XAttribute("FirstName", CStr(cust.Attribute("FirstName"))),
                        New XAttribute("LastName", CStr(cust.Attribute("LastName"))),
                        New XText(cust.Value))
                    %>
                </PositiveBalances>
                <NegativeBalances>
                    <%= From cust In x_all.Descendants("Customer")
                        Where CDec(cust.Value) < 0
                        Order By CDec(cust.Value) Descending
                        Select New XElement("Customer",
                        New XAttribute("FirstName", CStr(cust.Attribute("FirstName"))),
                        New XAttribute("LastName", CStr(cust.Attribute("LastName"))),
                        New XText(cust.Value))
                    %>
                </NegativeBalances>
            </AllCustomers>
        txt &= "LINQ Grouped Customers:" & vbCrLf & separated.ToString() & vbCrLf & vbCrLf

        ' Select all customers.
        txt &= "All Customers:" & vbCrLf
        Dim select_all = From cust In x_all.Descendants("Customer")
            Order By CDec(cust.Value)
            Select FName = cust.Attribute("FirstName").Value,
                   LName = cust.Attribute("LastName").Value,
                   Balance = cust.Value
        For Each c In select_all
            txt &= Space$(4) & c.FName & " " & c.LName & vbTab & FormatCurrency(c.Balance) & vbCrLf
        Next c
        txt &= vbCrLf

        ' Select only those with negative balances.
        txt &= "Customers with negative balances:" & vbCrLf
        Dim x_neg = From cust In x_all.Descendants("Customer")
            Where CDec(cust.Value) < 0
            Select FName = cust.Attribute("FirstName").Value,
                   LName = cust.Attribute("LastName").Value,
                   Balance = cust.Value
        For Each c In x_neg
            txt &= Space$(4) & c.FName & " " & c.LName & vbTab & FormatCurrency(c.Balance) & vbCrLf
        Next c
        txt &= vbCrLf

        txtResults.Text = txt
        txtResults.Select(0, 0)
    End Sub
End Class

Public Class Customer
    Public FirstName As String
    Public LastName As String
    Public Balance As Decimal
    Public Overrides Function ToString() As String
        Return FirstName & " " & LastName & " (" & Balance & ")"
    End Function
End Class
